function dataout = PIcalc(datain, envOuter, envInner, p)
% PICALC calculates PI: (phyto) plankton (community) index
%
% The transformed abundances of two lifeforms are, notionally
%   plotted against each other, 
%   giving points in a 2-D, Cartesian, lifeform space.
% Outer and inner reference envelopes (defining a 2-D doughnut)
%   are superimposed on this plot.
% PI = number of points outside envelopes, divided by total points
% Significance calculated by exact bionomial and chi-square tests,
%   using a supplied expectation (should be that used to make envelope)
%
% input
%   datain : array, 4 columns, in which
%       col 1 = data, year.dec - not used
%       col 2 = Y1, lifeform 1 transformed abundance = X co-ordinate
%       col 3 = Y2, lifeform 2 transformed abundance = Y co-ordinate
%       col 4 = month - is not used
%   envOuter : array, 2 columns, giving X and Y co-ordinates of
%           outer reference envelope
%   envInner : array, 2 columns, giving X and Y co-ordinates of
%           inner reference envelope 
%               (surrounding the hole in the doughnut)
%   p = scalar, proportion of points that were included between 
%           the pair of reference envelopes
%           (excluded proportion is (1-p)/2 beyond outer envelope and
%               the same within inner envelope)
% output
%   dataout : vector, in which
%       (1) = PI value
%       (2) = total number of points (data rows)
%       (3) = number of points outside envelope
%       (4) = probability (exact binomial test)
%       (5) = value of chi-quare (1 df)

%
% history and authorship:
%   Made by PT, 10 Aug 08, drawing on script PCI1ED2
%       as function dataout = PCIcalc(datain, envOuter, envInner)
%       which used a default excluded proportion, typically 0.95
%   PCIcalc2 made by PT, 9 August 2009, to use a supplied exclusion prop.
%   PIcalc converted from PCIcalc2 by PT, 3 October 2015, with
%       more accurate chi-square and improved commenting and names
% ======================================================

%% PRELIMINARY - check and set the expected proportions of points
% p should be in range 0.5 to 1.0; adjust if not
if p < 0.5 || p > 1.0, p = 0.95; end
q = 1-p; % the excluded proportion
%
%% STEP 1: identify and count points inside and outside envelopes
%   using the Matlab function INPOLYGON
% Identify points outside the outer envelope
PointsNewInsideOuter = ...
    inpolygon(datain(:,2), datain(:,3), envOuter(:,1), envOuter(:,2));
    % the array POINTSNEWINSIDEOUTER matches the rows of DATAIN, and now
    % contains a '0' for a point whose co-ordinates place it outside
    % the envelope and a '1' if it is inside or on the envelope
% Identify points inside the inner envelope
[PointsNewOutsideInner(:,1) PointsNewOutsideInner(:,2)] = ...
    inpolygon(datain(:,2), datain(:,3), envInner(:,1), envInner(:,2));
    % the array POINTSNEWOUTSIDEINNER matches the rows of DATAIN, and now
    % contains, in column 1, 
    % a '0' for a point whose co-ordinates place it outside
    % the (inner) envelope and a '1' if it is inside or on the envelope
    % contains, in column 2,
    % a '0' for a point inside or outside the (inner) envelope
    % and a '1' for a point on the (inner) envelope
% Identify all points between (or on) the two envelopes
    % i.e., all those for which POINTSNEWINSIDEOUTER = 1 and
    % either POINTSNEWOUTSIDEINNER(1) = 0 or POINTSNEWOUTSIDEINNER(2) = 1
PointsInsideNew = PointsNewInsideOuter & ...
    (~PointsNewOutsideInner(:,1) | PointsNewOutsideInner(:,2));
% Get the number of included points
NinsideNew = length(find(PointsInsideNew>0.5));
% Get the total number of new points and the total of points outside
NtotalNew = length(datain(:,2));
NoutsideNew = NtotalNew- NinsideNew;

% 
%% STEP 2: Calculate the Plankton Index (PI) and associated statistics
% ===============
PI = NinsideNew/NtotalNew;
% ===============
%
% Calculate the exact binomial probability for NOUTSIDENEW given 
% NTOTALNEW and P. Use the Matlab function NCHOOSEK.
% FROM MATLAB HELP: 
%   C = nchoosek(n,k) where n and k are nonnegative integers, 
%   returns n!/((nk)! k!). This is the number of combinations 
%   of n things taken k at a time. 
% Writing NoN for the value in NOUTSIDENEW 
%   NiN for NINSIDENEW and N for NTOTALNEW,
% 	the probability of the particular outcome NoN given N, p and q, is
%       (N!/(NiN!*NoN!)) * q^NoN * p^NiN
%   But it is also necessary to include the probabilities of more
%   extreme events - i.e. those with more outside points (counted by NoN)
%   and fewer inside points (counted by NiN). Thus the function
%   iterates through (N!/((N-k)!*k!)) * q^k * p^(N-k), for
%   k = NoN to N.
% NB: it might be efficient to halt after a few iterations, since 
%   higher values of k are likely to be very improbable. But I don't. 
warning off; % in order to ignore the following: FROM MATLAB HELP:
%   When C = nchoosek(n,k) has a large coefficient, 
%   a warning will be produced indicating possible inexact results. 
%   In such cases, the result is only accurate to 15 digits 
%   for double-precision inputs, or 8 digits for single-precision inputs.
% Note: the binomial calculation scheme was rearranged 3 Oct 2015 from the
%   version in PCICALC2, but this was not intended to change the results
% Now for the iteration. Might as well calculate all outcomes, so
outcomes = zeros(1,NtotalNew+1); % Matlab array numbering starts at [1]
for k = 0:NtotalNew,
    j = NtotalNew-k;
	outcomes(k+1)= nchoosek(NtotalNew,k) * p^j * q^k;
end;
% ========
BinProb = sum(outcomes(NoutsideNew+1:NtotalNew+1));
% ========
warning on;	
%
% Chi-square - revised 3 Oct 15 to include observed inside point numbers
%   (which were omitted in PCIcalc2: however, effect is small, because
%       expectation is (usually) high)
EIn = NtotalNew * p;
EOut = NtotalNew * q;
ChiSquare=(NinsideNew - EIn)^2/EIn + (NoutsideNew - EOut)^2/EOut;
%
%% STEP 3: Return values
dataout(1) = PI;
dataout(2) = NtotalNew;
dataout(3) = NoutsideNew;
dataout(4) = BinProb;
dataout(5) = ChiSquare;

% end of function